<?php

namespace WebSocket\Application;

/**
 * IRC Application
 * 
 * @author Jérome Freyre <jerome.freyre@heig-vd.ch>
 */
class IRCApplication extends Application
{
	/**
	 * Represent the client list
	 */
	private $clients = array();

	/**
	 * Is a flag that indicate if a user has been updated
	 */
	private $diffOnClients = false;

	/**
	 * Will be called when a client connect.
	 */
	public function onConnect($client) {
		$this->clients[] = $client;
	}

	/**
	 * Will be called when a client disconnect.
	 */
	public function onDisconnect($client) {
		$key = array_search($client, $this->clients);
		if ($key) {
			unset($this->clients[$key]);
		}
	}

	/**
	 * Will be called when a client send data
	 */
	public function onData($data, $client) {
		// Decode received data
		$data = json_decode($data);
		
		// Based on type
		switch($data->type) {
			case 'message'	: $this->broadcastMsg($data->data, $client);
								break;
								
			case 'username' : $this->updateUsername($data->data, $client);
								break;
		}
	}

	/**
	 * Called when a client update his username
	 */
	public function updateUsername($username, $client) {
		// Search the client
		$key = array_search($client, $this->clients);
		// Update only valid client
		if ($key) {
			$this->log($client->toString().' is now known as '. $username);
			$this->clients[$key]->username = $username;
			// Activate the flag
			$this->diffOnClients = true;
		}
	}
	
	/**
	 * Called to broadcast a message
	 */
	public function broadcastMsg($data, $client) {
		// Create object that will be broadcasted
		$obj = (object) null;
		$obj->type = 'message';
		$obj->data->from = $client->toString();
		$obj->data->msg = $data;
		
		// Broadcasting the message
		foreach ($this->clients as $sendto) {
			if ($sendto !== $client) // Do not resend at the sender
				$sendto->send(json_encode($obj));
		}
	}
	
	/**
	 * Is always called
	 * @note Use with caution!
	 */
	public function onTick() {
		// Will do something only if needed
		if ($this->diffOnClients) {
			
			// Create object that will be broadcasted
			$obj = (object) null;
			$obj->type = 'users-list';
			$obj->data = array();
			
			// Create client array
			foreach ($this->clients as $client) {
				if ($client->toString() != null)
					$obj->data[] = $client->toString();
			}
			
			// Broadcast the client array
			foreach ($this->clients as $sendto) {
				$sendto->send(json_encode($obj));
			}
			
			// Deactivate the flag
			$this->diffOnClients = false;
		}
	}
}